//! Template project for Flipper Zero.
//! This app prints "Hello, Rust!" to the console then exits.

#![no_main]
#![no_std]

// Required for panic handler
extern crate flipperzero_rt;

#[cfg(feature = "alloc")]
extern crate flipperzero_alloc;

use core::{
    ffi::{c_char, c_void, CStr},
    ptr,
    time::Duration,
};
use flipperzero::gui::canvas::*;
use flipperzero::{dialogs, furi::thread::sleep};
use flipperzero_rt::{entry, manifest};
use flipperzero_sys as sys;

// Define the FAP Manifest for this application
manifest!(
    name = "Resistor Decoder",
    app_version = 1,
    has_icon = true,
    // See https://github.com/flipperzero-rs/flipperzero/blob/v0.7.2/docs/icons.md for icon format
    icon = "rustacean-10x10.icon",
);

// Define the entry function
entry!(main);

/*
// Entry point
fn main(_args: *mut u8) -> i32 {
    let mut dialog = dialogs::DialogsApp::open();

    let mut message = dialogs::DialogMessage::new();
    message.set_header(
        CStr::from_bytes_with_nul(b"Make your move!\0").unwrap(),
        0,
        0,
        Align::Left,
        Align::Top,
    );
    message.set_text(
        CStr::from_bytes_with_nul(b"Choose one of the following:\0").unwrap(),
        0,
        26,
        Align::Left,
        Align::Top,
    );
    message.set_buttons(
        Some(CStr::from_bytes_with_nul(b"Rock\0").unwrap()),
        Some(CStr::from_bytes_with_nul(b"Paper\0").unwrap()),
        Some(CStr::from_bytes_with_nul(b"Scissors\0").unwrap()),
    );

    let button = dialog.show_message(&message);

    match button {
        dialogs::DialogMessageButton::Left => dialogs::alert("You chose Rock..."),
        dialogs::DialogMessageButton::Center => dialogs::alert("You chose Paper..."),
        dialogs::DialogMessageButton::Right => dialogs::alert("You chose Scissors..."),
        dialogs::DialogMessageButton::Back => dialogs::alert("You chose not to play..."),
    }

    dialogs::alert("... but dolphins can't play rock paper scissors anyways :)");

    0
}
*/

const RECORD_GUI: *const c_char = sys::c_string!("gui");
const FULLSCREEN: sys::GuiLayer = sys::GuiLayer_GuiLayerFullscreen;

pub unsafe extern "C" fn draw_callback(canvas: *mut sys::Canvas, _context: *mut c_void) {
    unsafe {
        sys::canvas_draw_str(canvas, 39, 31, sys::c_string!("Hello, Rust!"));
    }
}

fn main(_args: *mut u8) -> i32 {
    unsafe {
        let view_port = sys::view_port_alloc();
        sys::view_port_draw_callback_set(view_port, Some(draw_callback), ptr::null_mut());

        let gui = sys::furi_record_open(RECORD_GUI) as *mut sys::Gui;
        sys::gui_add_view_port(gui, view_port, FULLSCREEN);

        sleep(Duration::from_secs(1));

        sys::view_port_enabled_set(view_port, false);
        sys::gui_remove_view_port(gui, view_port);
        sys::furi_record_close(RECORD_GUI);
        sys::view_port_free(view_port);
    }

    0
}
